import React, { useEffect, useRef, useState } from 'react';
import { useAuthorizationCache, useQuerybuilderGraph, useSessionCache } from '@graphpolaris/shared/lib/data-access';
import { useAppDispatch, useML, useQuerybuilderSettings } from '@graphpolaris/shared/lib/data-access/store';
import { resetGraphQueryResults, queryingBackend } from '@graphpolaris/shared/lib/data-access/store/graphQueryResultSlice';
import { Query2BackendQuery, QueryBuilder, QueryMultiGraph } from '@graphpolaris/shared/lib/querybuilder';
import { Schema } from '@graphpolaris/shared/lib/schema/panel';
import { Navbar } from '../components/navbar/navbar';
import { VisualizationPanel } from '@graphpolaris/shared/lib/vis/panel';
import { Resizable } from '@graphpolaris/shared/lib/components/Resizable';
import { DashboardAlerts } from '@graphpolaris/shared/lib/data-access/authorization/dashboardAlerts';
import { EventBus } from '@graphpolaris/shared/lib/data-access/api/eventBus';
import Onboarding from '../components/onboarding/onboarding';
import { wsQueryRequest } from '@graphpolaris/shared/lib/data-access/api/wsQuery';
import { URLParams, setParam } from '@graphpolaris/shared/lib/data-access/api/url';

export type App = {
  load?: string;
};

export function App(props: App) {
  const auth = useAuthorizationCache();
  const query = useQuerybuilderGraph() as QueryMultiGraph;
  const ml = useML();
  const session = useSessionCache();
  const dispatch = useAppDispatch();
  const queryBuilderSettings = useQuerybuilderSettings();

  const runQuery = () => {
    if (session?.currentSaveState && query) {
      if (query.nodes.length === 0) {
        dispatch(resetGraphQueryResults());
      } else {
        dispatch(queryingBackend());
        wsQueryRequest(Query2BackendQuery(session.currentSaveState, query, queryBuilderSettings, ml));
      }
    }
  };

  useEffect(() => {
    if (props.load) {
      setParam(URLParams.saveState, props.load);
    }
  }, [props]);

  const [authCheck, setAuthCheck] = useState(false);

  return (
    <div className="h-screen w-screen overflow-clip">
      <EventBus
        onRunQuery={runQuery}
        onAuthorized={() => {
          setAuthCheck(true);
        }}
      />
      <Onboarding />
      <DashboardAlerts />
      <div className={'h-screen w-screen ' + (!auth.authorized ? 'blur-sm pointer-events-none ' : '')}>
        <div className="flex flex-col h-screen max-h-screen relative">
          <aside className="h-auto w-auto">
            <Navbar />
          </aside>
          <main className="flex w-screen h-[calc(100%-4.2rem)]">
            <Resizable divisorSize={3} horizontal={true} defaultProportion={0.33}>
              <div className="h-full w-full panel">
                <Schema auth={authCheck} />
              </div>
              <div className="h-full w-full">
                <Resizable divisorSize={3} horizontal={false}>
                  <div className="w-full h-full panel">
                    <VisualizationPanel />
                  </div>
                  <div className="w-full h-full panel">
                    <QueryBuilder onRunQuery={runQuery} />
                  </div>
                </Resizable>
              </div>
            </Resizable>
          </main>
        </div>
      </div>
    </div>
  );
}

export default App;
